//
//  TableControl.m
//  Local TeX
//
//  Created by Richard Koch on 2/16/14.
//  Copyright (c) 2014 Richard Koch. All rights reserved.
//

#import <CoreServices/CoreServices.h>
#import "TableControl.h"

@implementation TableControl

- (void)fillTable
{
    [myLTModel setupData];
    myLTModel.changed = NO;
    self.loaded = YES;
    [theTable reloadData];
}


- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
    if (self.loaded)
        return [myLTModel.Distributions count];
    else
        return 0;
}


- (NSView *)tableView:(NSTableView *)tableView
   viewForTableColumn:(NSTableColumn *)tableColumn
                  row:(NSInteger)row
{
    NSTableCellView *result;
    NSDictionary *theItem;
    BOOL is64Bit;
    NSString *theName;
    
    // Retrieve to get the @"MyView" from the pool or,
    // if no version is available in the pool, load the Interface Builder version
    
    // Set the stringValue of the cell's text field to the nameArray value at row
    //   if ([tableColumn.identifier isEqualToString: @"First"])
    
    theItem = [myLTModel.Distributions objectAtIndex: row];
    
    if ([tableColumn.identifier isEqualToString: @"First"])
            {
            result = [tableView makeViewWithIdentifier:@"MyTeXDistribution" owner:self];
            theName = [theItem objectForKey: @"Name"];
            result.textField.stringValue = theName;
            if ([theName isEqualToString: myLTModel.activeDistributionName]) {
                [theTable selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
            }
        }
    else if ([tableColumn.identifier isEqualToString: @"Second"])
            {
                result = [tableView makeViewWithIdentifier:@"MyProcessor" owner:self];
                is64Bit = [[theItem objectForKey: @"Active64"] boolValue];
                if (is64Bit)
                    result.textField.stringValue = @"Intel 64";
                else
                    result.textField.stringValue = @"Universal";
            }

    return result;
}


- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
{
    NSDictionary *theItem;
    NSURL *theURL;
    NSFileManager *theFileManager;
    NSString* theLinkPath;
    NSString* theName;
    NSString* oldName;
    
    oldName = myLTModel.activeDistributionName;
    
    NSInteger theRow = [theTable selectedRow];
    if (theRow < 0) {
        myLTModel.activeDistributionName = @"Unknown";
       // theFileManager = [NSFileManager defaultManager];
       // theLinkPath = [@"~/Library/TeX/LocalTeX/texbin" stringByExpandingTildeInPath];
       // [theFileManager removeItemAtPath: theLinkPath error: NULL];
       // theLinkPath = [@"~/Library/TeX/LocalTeX/texroot" stringByExpandingTildeInPath];
       //  [theFileManager removeItemAtPath: theLinkPath error: NULL];
         }
    if (theRow >= 0)
        {
        theItem = [myLTModel.Distributions objectAtIndex: theRow];
        if ([[theItem objectForKey:@"Active64"] boolValue] == YES)
            theURL = [theItem objectForKey: @"PathTo64"];
        else
            theURL = [theItem objectForKey: @"PathTo32"];
        theLinkPath = [@"~/Library/TeX/LocalTeX/texbin" stringByExpandingTildeInPath];
        theFileManager = [NSFileManager defaultManager];
        [theFileManager removeItemAtPath: theLinkPath error: NULL];
        [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: [theURL path] error: NULL];
        
        theLinkPath = [@"~/Library/TeX/LocalTeX/texroot" stringByExpandingTildeInPath];
        theURL = [theItem objectForKey: @"PathToDist"];
        [theFileManager removeItemAtPath: theLinkPath error: NULL];
        [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: [theURL path] error: NULL];

        theLinkPath = [@"~/Library/TeX/LocalTeX/texdist" stringByExpandingTildeInPath];
        theURL = [theItem objectForKey: @"PathToTexDist"];
        if (theURL == nil)
            {
            [theFileManager removeItemAtPath: theLinkPath error: NULL];
            }
        else {
            [theFileManager removeItemAtPath: theLinkPath error: NULL];
            [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: [theURL path] error: NULL];
            }
        
        theName = [theItem objectForKey: @"Name"];
        myLTModel.activeDistributionName = theName;
        [myLTModel.extrasDictionary removeObjectForKey:@"ActiveName"];
        [myLTModel.extrasDictionary setObject: theName forKey:@"ActiveName"];
        }

    
    if (! [oldName isEqualToString:myLTModel.activeDistributionName])
        myLTModel.changed = YES;
        [myLTModel writeDefaults];
 }



- (IBAction)addDistribution:(id)sender
{
    
        [NSApp beginSheet:newDistributionPanel
           modalForWindow:[theTable window]
            modalDelegate:self
           didEndSelector:nil
              contextInfo:nil];
    
}


- (void)setNewDefault: (NSString *)program forPreference: (NSString *)preference toValue: (NSString *)value
{
    NSTask *myTask = [[NSTask alloc] init];
    [myTask setLaunchPath:@"/usr/bin/defaults"];
    [myTask setArguments: [NSArray arrayWithObjects: @"write", program, preference, value, nil]];
    [myTask launch];
}
    
    
- (IBAction)removeDistribution:(id)sender
{
    [myRemoveDistTable runRemoveDistributionPanel];
}

/*
 Select a default distribution and set the following links to point there:
    ~/Library/TeX/LocalTeX/texbin
    ~/Library/TeX/LocalTeX/texroot
    ~/Library/TeX/LocalTeX/texdist
 Set GUI paths to point to ~/Library/TeX/LocalTeX/texbin or ~/Library/TeX/LocalTeX/texbin/program
*/


- (IBAction)configureForLocalPrefPane:(id)sender
{
    NSString                *theLocalLink;
    NSString                *theShellStartupFile;
    NSString                *theContents, *theContents1, *theContents2;
    NSData                  *theData;
    NSArray                 *runningApps;
    NSRunningApplication    *anApp;
    int                     i;
    BOOL                    AppIsRunning;
    NSAlert                 *myAlert;
    NSFileManager           *theFileManager;
    
    AppIsRunning = NO;
    
    runningApps = [[NSWorkspace sharedWorkspace] runningApplications];
    for (i = 0; i < runningApps.count; i++) {
        anApp = (NSRunningApplication *)runningApps[i];
        if (
            ([anApp.bundleIdentifier  isEqual: @"com.googlecode.mactlmgr.tlu"]) ||
            ([anApp.bundleIdentifier  isEqual: @"TeXShop"]) ||
            ([anApp.bundleIdentifier  isEqual: @"fr.chachatelier.pierre.LaTeXiT"]) ||
            ([anApp.bundleIdentifier  isEqual: @"edu.ucsd.cs.mmccrack.bibdesk"])
            )
        AppIsRunning = YES;
        }
    
   if (AppIsRunning) {
        
       myAlert = [NSAlert alertWithMessageText:@"Quit Running Application" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"BibDesk, LaTeXiT, TeX Live Utility, and TeXShop cannot be running while configuring for the Local Pane."];
        [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
       
       return;
    }
    
    theLocalLink = [@"~/Library/TeX/LocalTeX/texbin" stringByExpandingTildeInPath];
    
    // TeXShop
    [self setNewDefault:@"TeXShop" forPreference: @"TetexBinPath" toValue: theLocalLink];
    // TeX Live Utility
    [self setNewDefault:@"com.googlecode.mactlmgr.tlu" forPreference: @"TLMTexBinPathPreferenceKey" toValue: theLocalLink];
    [self setNewDefault:@"edu.ucsd.cs.mmccrack.bibdesk" forPreference:@"BibTeX Binary Path"
                toValue: [theLocalLink stringByAppendingString:@"/bibtex"]];
    [self setNewDefault:@"edu.ucsd.cs.mmccrack.bibdesk" forPreference:@"TeX Binary Path"
                toValue: [theLocalLink stringByAppendingString:@"/pdflatex"]];
    
    
    // LaTeXiT
    /*
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"latexPath"
                toValue: [theLocalLink stringByAppendingString:@"/latex"]];
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"pdfLatexPath"
                toValue: [theLocalLink stringByAppendingString:@"/pdflatex"]];
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"xeLatexPath"
                toValue: [theLocalLink stringByAppendingString:@"/xelatex"]];
     */
    
    theFileManager = [NSFileManager defaultManager];
    theShellStartupFile = [@"~/.profile" stringByExpandingTildeInPath];
    if ([theFileManager fileExistsAtPath:theShellStartupFile])
        return;
    theShellStartupFile = [@"~/.bash_login" stringByExpandingTildeInPath];
    if ([theFileManager fileExistsAtPath:theShellStartupFile])
        return;
    theShellStartupFile = [@"~/.bash_profile" stringByExpandingTildeInPath];
    if ([theFileManager fileExistsAtPath:theShellStartupFile])
        return;
    theContents1 = @"# Added by LocalTeX Preference Pane";
    theContents2 = [[@"export PATH=\"" stringByAppendingString: theLocalLink] stringByAppendingString: @"\":$PATH"];
    theContents = [NSString stringWithFormat:@"\n%@\n%@\n\n", theContents1, theContents2];
    theData = [theContents dataUsingEncoding: NSUTF8StringEncoding];
    [theFileManager createFileAtPath: theShellStartupFile contents: theData attributes: nil];
}

/*
 Set ~/Library/TeX/LocalTeX/texbin --> /usr/texbin for shell access
 Remove ~/Library/TeX/LocalTeX/texroot and ~/Library/TeX/LocalTeX/texdist links
 Set GUI paths to point to /usr/texbin or /usr/texbin/program
*/

- (IBAction)configureForOldPrefPane:(id)sender
{
    NSFileManager   *theFileManager;
    NSString        *theLinkPath, *theGlobalLink;
    NSArray                 *runningApps;
    NSRunningApplication    *anApp;
    int                     i;
    BOOL                    AppIsRunning;
    NSAlert                 *myAlert;
    
    
    AppIsRunning = NO;
    
    runningApps = [[NSWorkspace sharedWorkspace] runningApplications];
    for (i = 0; i < runningApps.count; i++) {
        anApp = (NSRunningApplication *)runningApps[i];
        if (
            ([anApp.bundleIdentifier  isEqual: @"com.googlecode.mactlmgr.tlu"]) ||
            ([anApp.bundleIdentifier  isEqual: @"TeXShop"]) ||
            ([anApp.bundleIdentifier  isEqual: @"fr.chachatelier.pierre.LaTeXiT"]) ||
            ([anApp.bundleIdentifier  isEqual: @"edu.ucsd.cs.mmccrack.bibdesk"])
            )
            AppIsRunning = YES;
    }
    
    if (AppIsRunning) {
        
        myAlert = [NSAlert alertWithMessageText:@"Quit Running Application" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"BibDesk, LaTeXiT, TeX Live Utility, and TeXShop cannot be running while configuring for the Old Pane."];
        [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
        
        return;
    }
    
    
    theFileManager = [NSFileManager defaultManager];
    theLinkPath = [@"~/Library/TeX/LocalTeX/texbin" stringByExpandingTildeInPath];
    [theFileManager removeItemAtPath: theLinkPath error: NULL];
 // [theFileManager createSymbolicLinkAtPath: theLinkPath withDestinationPath: @"/usr/texbin" error: NULL];
    theLinkPath = [@"~/Library/TeX/LocalTeX/texroot" stringByExpandingTildeInPath];
    [theFileManager removeItemAtPath: theLinkPath error: NULL];
    theLinkPath = [@"~/Library/TeX/LocalTeX/texdist" stringByExpandingTildeInPath];
    [theFileManager removeItemAtPath: theLinkPath error: NULL];
    
    theGlobalLink = @"/usr/texbin";
    
    // TeXShop
    [self setNewDefault:@"TeXShop" forPreference: @"TetexBinPath" toValue: theGlobalLink];
    // TeX Live Utility
    [self setNewDefault:@"com.googlecode.mactlmgr.tlu" forPreference: @"TLMTexBinPathPreferenceKey" toValue: theGlobalLink];
    // BibDesk
    [self setNewDefault:@"edu.ucsd.cs.mmccrack.bibdesk" forPreference:@"BibTeX Binary Path"
                toValue: [theGlobalLink stringByAppendingString:@"/bibtex"]];
    [self setNewDefault:@"edu.ucsd.cs.mmccrack.bibdesk" forPreference:@"TeX Binary Path"
                toValue: [theGlobalLink stringByAppendingString:@"/pdflatex"]];
    // LaTeXiT
    /*
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"latexPath"
                toValue: [theGlobalLink stringByAppendingString:@"/latex"]];
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"pdfLatexPath"
                toValue: [theGlobalLink stringByAppendingString:@"/pdflatex"]];
    [self setNewDefault:@"fr.chachatelier.pierre.LaTeXiT" forPreference:@"xeLatexPath"
                toValue: [theGlobalLink stringByAppendingString:@"/xelatex"]];
     */

    
}




- (IBAction)okPushed:(id)sender
{
    NSString *newDistributionName;
    NSString *newDistributionPath;
    NSString *newBinaryPath;
    
    newDistributionName = [distributionName stringValue];
    newDistributionPath = [distributionPath stringValue];
    newBinaryPath = [binaryPath stringValue];
    
    [NSApp endSheet: newDistributionPanel];
    [newDistributionPanel orderOut:sender];
    
    // myFileManager = [NSFileManager defaultManager];
    //if ([myFileManager fileExistsAtPath: [newBinaryPath stringByAppendingString: @"/pdftex"]])
    if ([myLTModel activeDistributionAtPath:newBinaryPath])
        {
        [myLTModel addExtraDistributionWithName:newDistributionName binaryPath: newBinaryPath rootPath:newDistributionPath];
        [myLTModel setupData];
        [self setLoaded: YES];
        [theTable reloadData];
        }
    else
        [self errorBinaryDirectory];
    
}

- (IBAction)cancelPushed:(id)sender
{
    [NSApp endSheet: newDistributionPanel];
    [newDistributionPanel orderOut:sender];
}

- (void)errorTeXDirectory
{
    NSAlert *myAlert;
    
    myAlert = [NSAlert alertWithMessageText:@"Cannot Create TeX Folder" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"~/Library/TeX exists as a file, so the TeX folder cannot be created."];
    [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
}

- (void)errorLocalTeXDirectory
{
    NSAlert *myAlert;
    
    myAlert = [NSAlert alertWithMessageText:@"Cannot Create LocalTeX Folder" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"~/Library/TeX/LocalTeX exists as a file, so the LocalTeX folder cannot be created."];
    [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
}


- (void)errorBinaryDirectory
{
    NSAlert *myAlert;
    
    myAlert = [NSAlert alertWithMessageText:@"Cannot Create New Distribution" defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:@"The proposed binary directory does not contain tex, pdftex, luatex, or xetex."];
    [myAlert beginSheetModalForWindow:[theTable window] modalDelegate: self didEndSelector: nil contextInfo: nil];
}

- (void)registerForDrags
{
    [distributionPath registerForDraggedTypes:[NSArray arrayWithObjects:
                                               NSURLPboardType, nil]];
    [binaryPath registerForDraggedTypes: [NSArray arrayWithObjects:
                                                NSURLPboardType, nil]];
}

@end
